/*
 *  GUI/Widgets/SearchFoldersList.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using Gtk;
using Gdk;
using Mono.Unix;
using Incollector.Searching;



namespace Incollector.GUI.Widgets {
	
	public class SearchFoldersList : Gtk.ComboBox {
	
		protected enum ColsEnum {
			Icon  = 0,
			Title = 1,
			ID    = 2
		};
		
		protected enum SpecialFoldersEnum {
			AllEntries    = -1,
			ManageEntries = -2,
			CreateEntry   = -3
		};
		
		
		protected int ActiveFolderItem = -1;
		protected SearchFoldersSidebar FoldersList;
		
		
		public delegate void ManageFoldersActivatedHandler ();
		public event ManageFoldersActivatedHandler ManageFoldersActivated;
		
		
		public SearchFoldersList (SearchFoldersSidebar FoldersList)
		{
			this.FoldersList = FoldersList;
			
			
			Gtk.ListStore store = new Gtk.ListStore (typeof (Gdk.Pixbuf), typeof (string), typeof (int));
			this.Model = store;
			this.WidthRequest = 170;
			this.Changed += new EventHandler (on_changed);
			
			store.SetSortFunc (0, this.Compare);
			store.SetSortColumnId (0, SortType.Ascending);
			
			Gtk.CellRendererPixbuf PixRend = new Gtk.CellRendererPixbuf ();
			PixRend.Xpad = 3;
			Gtk.CellRendererText TxtRend = new Gtk.CellRendererText ();
			TxtRend.Xpad = 3;
			
			this.Clear ();
			this.PackStart (PixRend, false);
			this.AddAttribute (PixRend, "pixbuf", (int) ColsEnum.Icon);
			this.PackStart (TxtRend, false);
			this.AddAttribute (TxtRend, "markup", (int) ColsEnum.Title);
			
			this.AddSpecialFolder (Icons.GetPixbuf (Icons.SHOW_ALL_TOPBAR), Catalog.GetString ("All the entries"), -1);
			this.AddSpecialFolder (null, "<b>" +Catalog.GetString ("Manage folders")+"</b>", -2);
			this.AddSpecialFolder (null, "<b>" +Catalog.GetString ("Create folder")+"</b>", -3);
			
			SearchFolders.Added += new SearchFolders.AddedHandler (OnSearchFolderAdded);
			SearchFolders.Activated += new SearchFolders.ActivatedHandler (OnSearchFoldersActivated);
			SearchFolders.Removed += new SearchFolders.RemovedHandler (this.OnSearchFolderRemoved);
			SearchFolders.Modified += new SearchFolders.ModifiedHandler (this.OnSearchFolderModified);
		}
		
		
		protected void on_changed (object o, EventArgs args)
		{
			if (this.ActiveFolderItem == this.Active)
				return;
			
			Gtk.TreeIter itr;
			this.GetActiveIter (out itr);
			int id = (int) (this.Model as ListStore).GetValue (itr, (int) ColsEnum.ID);
			
			if (id == (int) SpecialFoldersEnum.ManageEntries) {
				if (ManageFoldersActivated != null)
					ManageFoldersActivated ();
				this.Active = this.ActiveFolderItem;
			
			} else if (id == (int) SpecialFoldersEnum.CreateEntry) {
				this.FoldersList.AddFolder ();
				this.Active = this.ActiveFolderItem;
			
			} else {
				this.ActiveFolderItem = this.Active;
				(new SearchFolder (id)).Activate ();
			}
		}
		
		
		protected int Compare (TreeModel model, TreeIter a, TreeIter b)
		{
			ListStore store = model as ListStore;
			int id1 = (int) (store.GetValue (a, (int) ColsEnum.ID));
			int id2 = (int) (store.GetValue (b, (int) ColsEnum.ID));
			
			if (id1 == -1)
				return -1;
			else if (id2 == -1)
				return 1;
			else if (id1 <= -2)
				return 1;
			else if (id2 <= -2)
				return -1;
			else
				return string.Compare (store.GetValue (a, (int) ColsEnum.Title) as string, 
									   store.GetValue (b, (int) ColsEnum.Title) as string);
		}
		
		
		protected void OnSearchFolderAdded (SearchFolder folder)
		{
			this.AddFolder (folder);
		}
		
		
		protected void OnSearchFolderRemoved (SearchFolder folder)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = this.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (folder.ID == (int) store.GetValue (itr, (int) ColsEnum.ID))
					store.Remove (ref itr);
		}
		
		
		protected void OnSearchFolderModified (SearchFolder folder)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = this.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (folder.ID == (int) store.GetValue (itr, (int) ColsEnum.ID))
					store.SetValue (itr, (int) ColsEnum.Title, folder.Name);
		}
		
		
		protected void OnSearchFoldersActivated (SearchFolder folder)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = this.Model as ListStore;
			
			int i = 0;
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr)) {
				if (folder.ID == (int) store.GetValue (itr, (int) ColsEnum.ID)) {
					this.Active = i;
					return;
				}
				
				i++;
			}
		}
		
		
		public void AddFolder (SearchFolder folder)
		{
			(this.Model as ListStore).AppendValues (new object [] {
								Icons.GetPixbuf (Icons.SEARCH_FOLDER_TOPBAR),
								folder.Name, folder.ID});
		}
		
		
		public void AddSpecialFolder (Gdk.Pixbuf icon, string name, int id)
		{
			(this.Model as ListStore).AppendValues (new object [] {icon, name, id});
		}
	}
}